@@ -1,10 +1,6 @@
 from typing import Optional, List
-import itertools
 
 from .lattice import Lattice
-from .physics import *
-from .result import *
-from .calc import *
 from .heterostructure_opt import *
 
 
@@ -24,7 +20,7 @@ class Heterostructure:
     lattice. `Heterostructure` describes a collection of crystals that build
     up the structure; to obtain the crystal lattice resulting from joining
     these crystals you must first define a way in which these crystals come
-    together (angles, etc. – use `opt` or `res` methods to do this),
+    together (angles, etc. – use `opt` or `calc` methods to do this),
     and the resulting lattice will be available as `supercell` attribute
     of the `Result` class (see documentation of the relevant methods)
     """
@@ -154,10 +150,10 @@ class Heterostructure:
 
         Parameters
         ----------
-        layers : List[
-            Lattice,res.set_vectors()
-            or (Lattice, float),
-            or (Lattice, (float, float, float))
+        layers : List[ \
+            Lattice,res.set_vectors() \
+            or (Lattice, float), \
+            or (Lattice, (float, float, float)) \
         ]
             List of layers to add to the structure.
             If list element is a tuple, the second element serves the same way
@@ -261,9 +257,28 @@ class Heterostructure:
 
     ### CALC METHODS
 
+    def __calc_strain_tensor_from_ADt(self, ADt: Matrix2x2, XBr: Matrix2x2) -> Matrix2x2:
+        """
+        Calculate strain tensor.
+
+        See docs of `Heterostructure.calc` for definition of strain tensor.
+
+        Parameters
+        ----------
+        ADt : Matrix 2x2
+        XBr : Matrix 2x2
+
+        Returns
+        -------
+        Matrix 2x2
+        """
+        BrDt, BtrBr, XXt = self.__get_basis_change_matrices(ADt, XBr)
+        return Heterostructure.__calc_strain_tensor(XBr, XXt)
+
     def calc(self,
              M=((1, 0), (0, 1)),
-             thetas=None) -> Result:
+             thetas=None,
+             calc_atoms=True) -> Result:
         """
         Calculates strain tensor and other properties of the system in given
         circumstances
@@ -324,25 +339,7 @@ class Heterostructure:
             thetas = [lay_desc[1][0] for lay_desc in self.__layers]
 
         ADt = np.array(M)
-        return self.__calc_aux(ADt, thetas)
-
-    def __calc_strain_tensor_from_ADt(self, ADt: Matrix2x2, XBr: Matrix2x2) -> Matrix2x2:
-        """
-        Calculate strain tensor.
-
-        See docs of `Heterostructure.calc` for definition of strain tensor.
-
-        Parameters
-        ----------
-        ADt : Matrix 2x2
-        XBr : Matrix 2x2
-
-        Returns
-        -------
-        Matrix 2x2
-        """
-        BrDt, BtrBr, XXt = self.__get_basis_change_matrices(ADt, XBr)
-        return Heterostructure.__calc_strain_tensor(XBr, XXt)
+        return self.__calc_aux(ADt, thetas, calc_atoms=calc_atoms)
 
     def __get_basis_change_matrices(self, ADt, XBr):
         """
@@ -367,7 +364,8 @@ class Heterostructure:
 
     def __calc_aux(self,
                    ADt: InMatrix2x2,
-                   thetas: List[Angle]) -> Result:
+                   thetas: List[Angle],
+                   calc_atoms: bool) -> Result:
         """
         Calculates strain tensor and other properties of the system in given
         circumstances
@@ -405,7 +403,14 @@ class Heterostructure:
         strain_tensors_wiki = [Heterostructure.__get_strain_tensor_wiki(XXt)
                                for XXt in XXts]
 
-        superlattice = self.__build_superlattice(XA @ ADt, ADt, BrDts)
+
+        superlattice = self.__build_superlattice(XA @ ADt, ADt, BrDts, calc_atoms)
+
+        A_atoms_per_el_cell = len(self.__substrate.atoms())
+        atom_count = np.abs(np.linalg.det(ADt) * A_atoms_per_el_cell)
+        for layer, BrDt in zip(self.__layers, BrDts):
+            B_atoms_per_el_cell = len(layer[0].atoms())
+            atom_count += np.round(np.abs(np.linalg.det(BrDt) * B_atoms_per_el_cell))
 
         return Result(
             self,
@@ -414,7 +419,8 @@ class Heterostructure:
             strain_tensors,
             strain_tensors_wiki,
             ADt,
-            ABtrs
+            ABtrs,
+            atom_count = atom_count
         )
 
     @staticmethod
@@ -446,7 +452,7 @@ class Heterostructure:
         return F
 
     def __build_superlattice(self, XDt: Matrix2x2, ADt: Matrix2x2,
-                             BrDts: List[Matrix2x2]) -> Lattice:
+                             BrDts: List[Matrix2x2], calc_atoms=True) -> Lattice:
         """
         Creates Lattice object describing superlattice (supercell)
         of the heterostructure
@@ -473,12 +479,13 @@ class Heterostructure:
         res.set_vectors(*full_XDt.T.tolist())
 
         # atoms
-        for i, lay, MDt in zip(range(len(lattices)), lattices, MDts):
-            # We stack the layers one above the other so atoms must be moved up
-            # by the sum of z-sizes of all elementary cells of layers below
-            z_offset = self.__get_z_offset(i)
-            z_offset /= full_XDt[2, 2]  # angstrom -> Direct coordinates
-            Heterostructure.__superlattice_add_atoms(res, lay, MDt, z_offset)
+        if calc_atoms:
+            for i, lay, MDt in zip(range(len(lattices)), lattices, MDts):
+                # We stack the layers one above the other so atoms must be moved up
+                # by the sum of z-sizes of all elementary cells of layers below
+                z_offset = self.__get_z_offset(i)
+                z_offset /= full_XDt[2, 2]  # angstrom -> Direct coordinates
+                Heterostructure.__superlattice_add_atoms(res, lay, MDt, z_offset)
 
         return res
 
@@ -577,7 +584,8 @@ class Heterostructure:
     def opt(self,
             max_el: int = 6,
             thetas: Optional[List[Optional[List[float]]]] = None,
-            algorithm: str = "moire"
+            algorithm: str = "fast",
+            log: bool = False
             ) -> Result:
         """
         Minimises strain, and calculates its value.
@@ -602,8 +610,15 @@ class Heterostructure:
             All angles are in radians.
 
         algorithm : str, optional
-            Default: "moire"
-            Accepted values: "moire", "brute"
+            Default: "fast"
+            Accepted values: "fast", "direct"
+
+        log : bool, optional
+            Default: False
+            If True, the Result will contain pandas.DataFrame in `log` attribute
+            The resulting DataFrame will contain information on supercell for every
+            combination of interlayer angles considered.
+            Requires `pandas` extra dependency
 
         Returns
         -------
@@ -612,11 +627,6 @@ class Heterostructure:
             For more information see documentation of `Result`
         """
 
-        # TODO: the whole routine is pretty complex;
-        #   maybe separate opt numerics into a class "MoireSolver" or stg.
-        #   so we don't have to deal with passing XA, XB etc. all the time,
-        #   and the mess in Heterostructure
-
         # Prepare ranges of theta values
         def is_iterable(x):
             try:
@@ -642,14 +652,18 @@ class Heterostructure:
         config = OptSolverConfig()
         config.max_el = max_el
         config.ord = (1, 1)
+        config.log = log
 
         XA, XBs = self.__get_lattice_matrices()
-        if algorithm == "moire":
-            thetas, ADt = MoireFinder(XA, XBs, thetas_in, config).solve()
-        else:  # "brute"
-            thetas, ADt = StrainOptimisator(XA, XBs, thetas_in, config).solve()
+        if algorithm == "fast":
+            thetas, ADt, additional = MoireFinder(XA, XBs, thetas_in, config).solve()
+        else:  # "direct"
+            thetas, ADt, additional = StrainOptimisator(XA, XBs, thetas_in, config).solve()
 
-        return self.calc(ADt, thetas)
+        res = self.calc(ADt, thetas)
+        if log:
+            res.log = additional["log"]
+        return res
 
     def __get_lattice_matrices(self):
         """
